/*
 * Created on Aug 20, 2003
 *
 */
package messaging;
import java.net.*;
import java.io.*;

/**
 * @author poladian
 *
 */
class NonYieldingSocketReaderThread extends Thread
{
	protected Socket 				m_socket;
	protected IMessageQueue		m_MessageQueue;
	protected boolean				m_bStopRunning = false;

	public NonYieldingSocketReaderThread(
						String name,
						Socket clientSoc,
						IMessageQueue readerQueue)
	{
		super("NYSocketReader-"+name);
		m_socket  = clientSoc;
		m_MessageQueue = readerQueue;
		this.start();
	}


/**
*   Overrides the run() method in java.lang.Thread.
*
*   Blocks on reading the input stream of the underlying socket,
*   does not check runtime for exceptions - thus, the thread
*   can become orphaned because of buggy code.
*
*   Queues the read messages to the IMessageQueue.
**/
	public void run()
	{
		try
		{
			InputStreamReader inStream = new InputStreamReader (m_socket.getInputStream());
			OutputStreamWriter outStream = new OutputStreamWriter (m_socket.getOutputStream());

			while ( !m_bStopRunning )
			{
				String strMessage = SocketProtocolMessageReader.readMessage(inStream);
				MessageWrapper wrap = new MessageWrapper();
				wrap._message = strMessage;
				try
				{
					m_MessageQueue.queueMessage(wrap);
				}
				catch (java.lang.InterruptedException interrupted)
				{
					break;
				}
			}
			try
			{
				System.out.println("@@@@ Stop signal received.");
				inStream.close();
				outStream.close();
				m_socket.close();
			}
			catch (Exception exc)
			{
				exc.printStackTrace();
			}

		}
		catch (java.io.IOException ioExc)
		{
			ioExc.printStackTrace();
			System.out.print("@@@@ Exception while reading from socket.  Dumping socket ... ");
			System.out.println(m_socket.getInetAddress().getHostAddress()
					+ ":" + m_socket.getPort());
		}
		catch (RuntimeException runtimeExc)
		{
			runtimeExc.printStackTrace();
/*			System.out.println("Listener address is: " + m_serverSocket.getInetAddress().getHostAddress()
					+ ":" + m_serverSocket.getLocalPort());

*/			System.out.println("Offending socket was: " + m_socket.getInetAddress().getHostAddress()
					+ ":" + m_socket.getPort());

		}
	}
}
